"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PaginationService = void 0;
const common_1 = require("@nestjs/common");
const pagination_constants_1 = require("./pagination.constants");
const sequelize_typescript_1 = require("sequelize-typescript");
const common_2 = require("./common");
const exceptions_1 = require("./exceptions");
let PaginationService = class PaginationService {
    constructor(options, sequelize) {
        this.options = options;
        this.sequelize = sequelize;
    }
    async findAllPaginate(model, options = {}, optionsSequelize = {}) {
        const modelName = model.name;
        if (!this.sequelize.models[modelName]) {
            throw new exceptions_1.ModelDoesNotRegisteredException(modelName);
        }
        const mergedOptions = (0, common_2.mergeDeep)({
            path: null,
            withDetails: true,
            limit: 50,
            page: 1,
            cls: common_2.PaginatedData,
        }, this.options, options);
        const { page, limit, url, path, withDetails } = mergedOptions;
        const offset = (page - 1) * limit;
        const data = await model.findAndCountAll({
            ...optionsSequelize,
            limit,
            offset,
        });
        const totalItemCount = data.count;
        const totalPages = totalItemCount > 0 ? Math.ceil(totalItemCount / limit) : 0;
        const nextPageNumber = page < totalPages ? page + 1 : null;
        const prevPageNumber = page > 1 ? page - 1 : null;
        let meta = {
            pageCount: totalPages,
            page,
            nextPage: nextPageNumber,
            prevPage: prevPageNumber,
        };
        if (withDetails) {
            const links = url
                ? this.createMetaLinks(url, path, page, limit, totalPages, nextPageNumber, prevPageNumber)
                : undefined;
            meta = {
                ...meta,
                limit,
                totalItems: totalItemCount,
                itemCount: data.rows.length,
                links,
            };
        }
        return new mergedOptions.cls(meta, data.rows);
    }
    createMetaLinks(baseUrl, path, page, limit, totalPages, nextPageNumber, prevPageNumber) {
        const selfUrl = new URL(baseUrl);
        if (path) {
            selfUrl.pathname = `${selfUrl.pathname.replace(/\/+$/g, '')}/${path.replace(/^\/+/g, '')}`;
        }
        const createPageUrl = (pageNumber) => {
            const pageUrl = new URL(selfUrl.toString());
            pageUrl.searchParams.set('page', pageNumber.toString());
            pageUrl.searchParams.set('limit', limit.toString());
            return pageUrl;
        };
        selfUrl.searchParams.set('page', page.toString());
        selfUrl.searchParams.set('limit', limit.toString());
        const nextPageUrl = nextPageNumber ? createPageUrl(nextPageNumber) : null;
        const prevPageUrl = prevPageNumber ? createPageUrl(prevPageNumber) : null;
        const firstPageUrl = createPageUrl(1);
        const lastPageUrl = totalPages > 0 ? createPageUrl(totalPages) : firstPageUrl;
        return {
            self: selfUrl,
            first: firstPageUrl,
            last: lastPageUrl,
            next: nextPageUrl,
            previous: prevPageUrl,
        };
    }
};
exports.PaginationService = PaginationService;
exports.PaginationService = PaginationService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, common_1.Inject)(pagination_constants_1.PAGINATION_MODULE_OPTIONS)),
    __metadata("design:paramtypes", [Object, sequelize_typescript_1.Sequelize])
], PaginationService);
